#!/usr/bin/perl
use warnings;
use strict;

use Time::HiRes qw(time);
use File::stat;

my ($server, $file, $dir) = @ARGV;

die "usage: $0 server file\n" unless $file && -f $file;

$dir //= "/tmp";
my $size = stat($file)->size;

my $ciphers = `ssh -Q cipher`;
my @ciphers = (
    "",
    split(/\s+/, $ciphers)
);

my @results;
for my $compress ("", "-C") {
    for my $cipher (@ciphers) {
        my @cmd = ("scp",
                   ($compress ? $compress : ()),
                   ($cipher ? ("-c", $cipher) : ()),
                   $file,
                   "$server:$dir/");
        print STDERR "@cmd\n";
        my $t0 = time;
        my $rc = system @cmd;
        if ($rc != 0) {
            print STDERR "@cmd failed";
            next;
        }
        my $t1 = time;
        my $dt = $t1 - $t0;
        printf STDERR ("%.1f seconds, %.1f MB/s\n",
                       $dt, $size / (1024 * 1024) / $dt);
        push @results, [ $cipher, $compress, $dt, $size / $dt ];
    }
}

@results = sort { $a->[2] <=> $b->[2] } @results;

print "<table>\n";
print "  <tr>";
print "<th>Cipher</th>";
print "<th>Compression</th>";
print "<th>Speed MiB/s</th>";
print "<th>elapsed (min:s)</th>";
print "</tr>\n";

for my $res (@results) {
    print "  <tr>";
    print "<th>$res->[0]</th>";
    print "<th>$res->[1]</th>";
    print "<td>", sprintf("%.1f", $res->[3] / (1024 * 1024)), "</td>";
    my $min = int($res->[2] / 60);
    my $sec = $res->[2] - $min * 60;
    print "<td>", sprintf("%02d:%04.1f", $min, $sec), "</td>";
    print "</tr>\n";
}
print "</table>\n";

